@@ -26,7 +26,7 @@
 
 static const int maxSimultaneousProcesses = 100;
 
-static const int maxMark = (1 << 20) - 1; // some versions of git-fast-import are buggy for larger values of maxMark
+static const int maxMark = (1 << 20) - 2; // some versions of git-fast-import are buggy for larger values of maxMark
 
 class ProcessCache: QLinkedList<Repository *>
 {
@@ -67,7 +67,7 @@ Repository::Repository(const Rules::Repository &rule)
 {
     foreach (Rules::Repository::Branch branchRule, rule.branches) {
         Branch branch;
-        branch.created = 0;     // not created
+        branch.created = 1;
 
         branches.insert(branchRule.name, branch);
     }
@@ -271,12 +271,15 @@ void Repository::closeFastImport()
 
 void Repository::reloadBranches()
 {
+    bool reset_notes = false;
     foreach (QString branch, branches.keys()) {
         Branch &br = branches[branch];
 
         if (br.marks.isEmpty() || !br.marks.last())
             continue;
 
+	reset_notes = true;
+
         QByteArray branchRef = branch.toUtf8();
         if (!branchRef.startsWith("refs/"))
             branchRef.prepend("refs/heads/");
@@ -285,6 +288,13 @@ void Repository::reloadBranches()
                         "\nfrom :" + QByteArray::number(br.marks.last()) + "\n\n"
                         "progress Branch " + branchRef + " reloaded\n");
     }
+
+    if (reset_notes &&
+	CommandLineParser::instance()->contains("add-metadata-notes")) {
+      fastImport.write("reset refs/notes/commits\nfrom :" +
+		       QByteArray::number(maxMark + 1) +
+		       "\n");
+    }
 }
 
 int Repository::markFrom(const QString &branchFrom, int branchRevNum, QByteArray &branchFromDesc)
@@ -554,7 +564,7 @@ QByteArray Repository::formatMetadataMessage(const QByteArray &svnprefix, int re
     return msg;
 }
 
-bool Repository::branchExists(const QString& branch) const\
+bool Repository::branchExists(const QString& branch) const
 {
     return branches.contains(branch);
 }
@@ -672,14 +682,16 @@ void Repository::Transaction::commitNote(const QByteArray &noteText, bool append
         message = "Appending Git note for current " + commitRef + "\n";
     }
 
-    QTextStream s(&repository->fastImport);
-    s << "commit refs/notes/commits" << endl
-      << "committer " << QString::fromUtf8(author) << ' ' << datetime << " -0000" << endl
-      << "data " << message.length() << endl
-      << message << endl
-      << "N inline " << commitRef << endl
-      <<  "data " << text.length() << endl
-      << text << endl;
+    QByteArray s("");
+    s.append("commit refs/notes/commits\n");
+    s.append("mark :" + QByteArray::number(maxMark + 1) + "\n");
+    s.append("committer " + QString::fromUtf8(author) + " " + QString::number(datetime) + " +0000" + "\n");
+    s.append("data " + QString::number(message.length()) + "\n");
+    s.append(message + "\n");
+    s.append("N inline " + commitRef + "\n");
+    s.append("data " + QString::number(text.length()) + "\n");
+    s.append(text + "\n");
+    repository->fastImport.write(s);
 
     if (commit.isNull()) {
         repository->setBranchNote(QString::fromUtf8(branch), text);
@@ -717,21 +729,17 @@ void Repository::Transaction::commit()
     br.commits.append(revnum);
     br.marks.append(mark);
 
-    {
-        QByteArray branchRef = branch;
-        if (!branchRef.startsWith("refs/"))
-            branchRef.prepend("refs/heads/");
-        QTextStream s(&repository->fastImport);
-        s.setCodec("UTF-8");
-        s << "commit " << branchRef << endl;
-        s << "mark :" << QByteArray::number(mark) << endl;
-        s << "committer " << QString::fromUtf8(author) << ' ' << datetime << " +0000" << endl;
+    QByteArray branchRef = branch;
+    if (!branchRef.startsWith("refs/"))
+        branchRef.prepend("refs/heads/");
 
-        s << "data " << message.length() << endl;
-    }
-
-    repository->fastImport.write(message);
-    repository->fastImport.putChar('\n');
+    QByteArray s("");
+    s.append("commit " + branchRef + "\n");
+    s.append("mark :" + QByteArray::number(mark) + "\n");
+    s.append("committer " + QString::fromUtf8(author) + " " + QString::number(datetime) + " +0000" + "\n");
+    s.append("data " + QString::number(message.length()) + "\n");
+    s.append(message + "\n");
+    repository->fastImport.write(s);
 
     // note some of the inferred merges
     QByteArray desc = "";
