@@ -44,8 +44,6 @@
 #include <svn_types.h>
 
 #include <QFile>
-#include <QLinkedList>
-#include <QPair>
 #include <QDebug>
 
 #include "repository.h"
@@ -79,6 +77,7 @@ public:
     QList<MatchRuleList> allMatchRules;
     RepositoryHash repositories;
     IdentityHash identities;
+    QString userdomain;
 
     SvnPrivate(const QString &pathToRepository);
     ~SvnPrivate();
@@ -130,6 +129,11 @@ void Svn::setIdentityMap(const IdentityHash &identityMap)
     d->identities = identityMap;
 }
 
+void Svn::setIdentityDomain(const QString &identityDomain)
+{
+    d->userdomain = identityDomain;
+}
+
 int Svn::youngestRevision()
 {
     return d->youngestRevision();
@@ -332,21 +336,29 @@ static int recursiveDumpDir(Repository::Transaction *txn, svn_fs_root_t *fs_root
     SVN_ERR(svn_fs_dir_entries(&entries, fs_root, pathname, pool));
     AprAutoPool dirpool(pool);
 
+    // While we get a hash, put it in a map for sorted lookup, so we can
+    // repeat the conversions and get the same git commit hashes.
+    QMap<QByteArray, svn_node_kind_t> map;
     for (apr_hash_index_t *i = apr_hash_first(pool, entries); i; i = apr_hash_next(i)) {
-        dirpool.clear();
         const void *vkey;
         void *value;
         apr_hash_this(i, &vkey, NULL, &value);
-
         svn_fs_dirent_t *dirent = reinterpret_cast<svn_fs_dirent_t *>(value);
-        QByteArray entryName = pathname + '/' + dirent->name;
-        QString entryFinalName = finalPathName + QString::fromUtf8(dirent->name);
+        map.insertMulti(QByteArray(dirent->name), dirent->kind);
+    }
 
-        if (dirent->kind == svn_node_dir) {
+    QMapIterator<QByteArray, svn_node_kind_t> i(map);
+    while (i.hasNext()) {
+        dirpool.clear();
+        i.next();
+        QByteArray entryName = pathname + '/' + i.key();
+        QString entryFinalName = finalPathName + QString::fromUtf8(i.key());
+
+        if (i.value() == svn_node_dir) {
             entryFinalName += '/';
             if (recursiveDumpDir(txn, fs_root, entryName, entryFinalName, dirpool) == EXIT_FAILURE)
                 return EXIT_FAILURE;
-        } else if (dirent->kind == svn_node_file) {
+        } else if (i.value() == svn_node_file) {
             printf("+");
             fflush(stdout);
             if (dumpBlob(txn, fs_root, entryName, entryFinalName, dirpool) == EXIT_FAILURE)
@@ -388,6 +400,7 @@ public:
     QList<MatchRuleList> allMatchRules;
     RepositoryHash repositories;
     IdentityHash identities;
+    QString userdomain;
 
     svn_fs_t *fs;
     svn_fs_root_t *fs_root;
@@ -436,6 +449,7 @@ int SvnPrivate::exportRevision(int revnum)
     rev.allMatchRules = allMatchRules;
     rev.repositories = repositories;
     rev.identities = identities;
+    rev.userdomain = userdomain;
 
     // open this revision:
     printf("Exporting revision %d ", revnum);
@@ -461,31 +475,41 @@ int SvnPrivate::exportRevision(int revnum)
 
 int SvnRevision::prepareTransactions()
 {
-    QLinkedList< QPair<svn_fs_path_change_t*, const char*> > sortedChanges;
     // find out what was changed in this revision:
     apr_hash_t *changes;
     SVN_ERR(svn_fs_paths_changed(&changes, fs_root, pool));
+
+    QMap<QByteArray, svn_fs_path_change_t*> map;
     for (apr_hash_index_t *i = apr_hash_first(pool, changes); i; i = apr_hash_next(i)) {
         const void *vkey;
         void *value;
         apr_hash_this(i, &vkey, NULL, &value);
         const char *key = reinterpret_cast<const char *>(vkey);
         svn_fs_path_change_t *change = reinterpret_cast<svn_fs_path_change_t *>(value);
-
-        // If we mix path deletions with path adds/replaces we might erase a branch after that it has been reset -> history truncated
-        if(change->change_kind == svn_fs_path_change_delete) {
-            sortedChanges.prepend( qMakePair(change, key) );
-        } else {
-            sortedChanges.append( qMakePair(change, key) );
+        // If we mix path deletions with path adds/replaces we might erase a
+        // branch after that it has been reset -> history truncated
+        if (map.contains(QByteArray(key))) {
+            // If the same path is deleted and added, we need to put the
+            // deletions into the map first, then the addition.
+            if (change->change_kind == svn_fs_path_change_delete) {
+                // XXX
+            }
+            fprintf(stderr, "\nDuplicate key found in rev %d: %s\n", revnum, key);
+            fprintf(stderr, "This needs more code to be handled, file a bug report\n");
+            fflush(stderr);
+            exit(1);
         }
+        map.insertMulti(QByteArray(key), change);
     }
 
-    QPair<svn_fs_path_change_t*, const char*> pair;
-    foreach (pair, sortedChanges) {
-        if (exportEntry(pair.second, pair.first, changes) == EXIT_FAILURE)
+    QMapIterator<QByteArray, svn_fs_path_change_t*> i(map);
+    while (i.hasNext()) {
+        i.next();
+        if (exportEntry(i.key(), i.value(), changes) == EXIT_FAILURE)
             return EXIT_FAILURE;
     }
-        return EXIT_SUCCESS;
+
+    return EXIT_SUCCESS;
 }
 
 int SvnRevision::fetchRevProps()
@@ -506,8 +530,8 @@ int SvnRevision::fetchRevProps()
         if (!svnauthor || svn_string_isempty(svnauthor))
             authorident = "nobody <nobody@localhost>";
         else
-            authorident = svnauthor->data + QByteArray(" <") +
-                          svnauthor->data + QByteArray("@localhost>");
+            authorident = svnauthor->data + QByteArray(" <") + svnauthor->data +
+                QByteArray("@") + userdomain.toUtf8() + QByteArray(">");
     }
     propsFetched = true;
     return EXIT_SUCCESS;
@@ -815,31 +839,42 @@ int SvnRevision::recurse(const char *path, const svn_fs_path_change_t *change,
         SVN_ERR(svn_fs_revision_root(&fs_root, fs, revnum - 1, pool));
 
     // get the dir listing
-    apr_hash_t *entries;
     svn_node_kind_t kind;
     SVN_ERR(svn_fs_check_path(&kind, fs_root, path, pool));
     if(kind == svn_node_none) {
         qWarning() << "WARN: Trying to recurse using a nonexistant path" << path << ", ignoring";
         return EXIT_SUCCESS;
+    } else if(kind != svn_node_dir) {
+        qWarning() << "WARN: Trying to recurse using a non-directory path" << path << ", ignoring";
+        return EXIT_SUCCESS;
     }
 
+    apr_hash_t *entries;
     SVN_ERR(svn_fs_dir_entries(&entries, fs_root, path, pool));
-
     AprAutoPool dirpool(pool);
+
+    // While we get a hash, put it in a map for sorted lookup, so we can
+    // repeat the conversions and get the same git commit hashes.
+    QMap<QByteArray, svn_node_kind_t> map;
     for (apr_hash_index_t *i = apr_hash_first(pool, entries); i; i = apr_hash_next(i)) {
         dirpool.clear();
         const void *vkey;
         void *value;
         apr_hash_this(i, &vkey, NULL, &value);
-
         svn_fs_dirent_t *dirent = reinterpret_cast<svn_fs_dirent_t *>(value);
         if (dirent->kind != svn_node_dir)
             continue;           // not a directory, so can't recurse; skip
+        map.insertMulti(QByteArray(dirent->name), dirent->kind);
+    }
 
-        QByteArray entry = path + QByteArray("/") + dirent->name;
+    QMapIterator<QByteArray, svn_node_kind_t> i(map);
+    while (i.hasNext()) {
+        dirpool.clear();
+        i.next();
+        QByteArray entry = path + QByteArray("/") + i.key();
         QByteArray entryFrom;
         if (path_from)
-            entryFrom = path_from + QByteArray("/") + dirent->name;
+            entryFrom = path_from + QByteArray("/") + i.key();
 
         // check if this entry is in the changelist for this revision already
         svn_fs_path_change_t *otherchange =
@@ -851,7 +886,7 @@ int SvnRevision::recurse(const char *path, const svn_fs_path_change_t *change,
         }
 
         QString current = QString::fromUtf8(entry);
-        if (dirent->kind == svn_node_dir)
+        if (i.value() == svn_node_dir)
             current += '/';
 
         // find the first rule that matches this pathname
@@ -861,11 +896,13 @@ int SvnRevision::recurse(const char *path, const svn_fs_path_change_t *change,
                                rev_from, changes, current, *match, matchRules, dirpool) == EXIT_FAILURE)
                 return EXIT_FAILURE;
         } else {
-            qDebug() << current << "rev" << revnum
-                     << "did not match any rules; auto-recursing";
-            if (recurse(entry, change, entryFrom.isNull() ? 0 : entryFrom.constData(),
-                        matchRules, rev_from, changes, dirpool) == EXIT_FAILURE)
-                return EXIT_FAILURE;
+            if (dirent->kind == svn_node_dir) {
+                qDebug() << current << "rev" << revnum
+                         << "did not match any rules; auto-recursing";
+                if (recurse(entry, change, entryFrom.isNull() ? 0 : entryFrom.constData(),
+                            matchRules, rev_from, changes, dirpool) == EXIT_FAILURE)
+                    return EXIT_FAILURE;
+            }
         }
     }
 
